Package: Rsamtools
Type: Package
Title: Binary alignment (BAM), FASTA, variant call (BCF), and tabix
        file import
Description: This package provides an interface to the 'samtools',
    'bcftools', and 'tabix' utilities for manipulating SAM (Sequence
    Alignment / Map), FASTA, binary variant call (BCF) and compressed
    indexed tab-delimited (tabix) files.
biocViews: DataImport, Sequencing, Coverage, Alignment, QualityControl
URL: https://bioconductor.org/packages/Rsamtools
Video:
        https://www.youtube.com/watch?v=Rfon-DQYbWA&list=UUqaMSQd_h-2EDGsU6WDiX0Q
BugReports: https://github.com/Bioconductor/Rsamtools/issues
Version: 2.20.0
License: Artistic-2.0 | file LICENSE
Encoding: UTF-8
Authors@R: c(
    person("Martin", "Morgan", role = "aut"),
    person("Hervé", "Pagès", role = "aut"),
    person("Valerie", "Obenchain", role = "aut"),
    person("Nathaniel", "Hayden", role = "aut"),
    person("Busayo", "Samuel", role = "ctb",
        comment = "Converted Rsamtools vignette from Sweave to RMarkdown / HTML."),
    person("Bioconductor Package Maintainer",
        email = "maintainer@bioconductor.org", role = "cre"))
Depends: methods, GenomeInfoDb (>= 1.1.3), GenomicRanges (>= 1.31.8),
        Biostrings (>= 2.47.6), R (>= 3.5.0)
Imports: utils, BiocGenerics (>= 0.25.1), S4Vectors (>= 0.17.25),
        IRanges (>= 2.13.12), XVector (>= 0.19.7), zlibbioc, bitops,
        BiocParallel, stats
Suggests: GenomicAlignments, ShortRead (>= 1.19.10), GenomicFeatures,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        TxDb.Hsapiens.UCSC.hg18.knownGene, RNAseqData.HNRNPC.bam.chr14,
        BSgenome.Hsapiens.UCSC.hg19, RUnit, BiocStyle, knitr
LinkingTo: Rhtslib (>= 2.99.1), S4Vectors, IRanges, XVector, Biostrings
LazyLoad: yes
SystemRequirements: GNU make
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/Rsamtools
git_branch: RELEASE_3_19
git_last_commit: ae36384
git_last_commit_date: 2024-04-30
Repository: Bioconductor 3.19
Date/Publication: 2024-04-30
NeedsCompilation: yes
Packaged: 2024-05-01 05:45:04 UTC; biocbuild
Author: Martin Morgan [aut],
  Hervé Pagès [aut],
  Valerie Obenchain [aut],
  Nathaniel Hayden [aut],
  Busayo Samuel [ctb] (Converted Rsamtools vignette from Sweave to
    RMarkdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Built: R 4.4.0; x86_64-apple-darwin23.2.0; 2024-05-07 07:35:52 UTC; unix
